unit Unit1;

interface

uses
  Windows, Forms, OpenGL, Classes, ExtCtrls, Messages;

type
  TMainForm = class(TForm)
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormResize(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    RC    : HGLRC;
    Angle : Integer;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
  end;

var
  MainForm: TMainForm;

implementation

uses Dialogs;

{$R *.DFM}

procedure TMainForm.FormCreate(Sender: TObject);
begin // initialize OpenGL and create a context for rendering
  RC:=CreateRenderingContext(Canvas.Handle,[opDoubleBuffered],32,0);
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin // destroy the rendering context
  DestroyRenderingContext(RC);
end;

procedure TMainForm.FormPaint(Sender: TObject);
begin // draw somthing useful
  ActivateRenderingContext(Canvas.Handle,RC); // make context drawable
  glClearColor(0,0.5,0.5,1); // background color of the context
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT); // clear background and depth buffer
  glMatrixMode(GL_MODELVIEW); // activate the transformation matrix
  glLoadIdentity;             // set it to initial state
  gluLookAt(0,0,6,0,0,-10,0,1,0); // set up a viewer position and view point
  //glTranslatef(0,0,0);
  glRotatef(4*Angle,0,1,0); // do rotation around axis (x:0;y:1;z:0)
  glRotatef(30,1,0,0);      // do another rotation for better view (accumulates to first rot.)
  glScalef(1,sin(Angle*pi/90),1); // simulate bumping
  glEnable(GL_DEPTH_TEST); // enable depth testing
  glBegin(GL_TRIANGLE_FAN); // draw a tetrahedron
    glColor3f(1,1,1);       // color of the upper vertex
    glVertex3f(0,1,0);      // coordinates of the upper vertex (= center of the fan)
    glColor3f(1,0,0);       // color left-front vertex
    glVertex3f(-1,0,1);     // coordinates of the left-front vertex
    glColor3f(0,1,0);       // right-front
    glVertex3f(1,0,1);      // right-front
    glColor3f(0,0,1);       // middle-back
    glVertex3f(0,0,-1);     // middle-back
    glColor3f(1,0,0);       // left-front again (close fan)
    glVertex3f(-1,0,1);     // left-front
  glEnd;                    // finish triangle fan
  SwapBuffers(Canvas.Handle); // copy back buffer to front
  DeactivateRenderingContext; // release control of context
end;

procedure TMainForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #27 then Application.Terminate;
end;

procedure TMainForm.FormResize(Sender: TObject);
begin // handle form resizing (viewport must be adjusted)
  wglMakeCurrent(Canvas.handle,RC); // another way to make context drawable
  glViewport(0,0,Width,Height); // specify a viewport (has not necessarily to be the entire window)
  glMatrixMode(GL_PROJECTION); // activate projection matrix
  glLoadIdentity;              // set initial state
  gluPerspective(35,Width/Height,1,100); // specify perspective params (see OpenGL.hlp)
  wglMakeCurrent(0,0); // another way to release control of context
  Refresh;             // cause redraw
end;

procedure TMainForm.Timer1Timer(Sender: TObject);
begin // do some animation
  Inc(Angle,1);
  if Angle >= 360 then Angle:=0;
  Repaint;
end;

procedure TMainForm.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin // avoid clearing the background (causes flickering and speed penalty)
  Message.Result:=1;
end;

end.

